/* This is the implementation file for the link class */

#include "link.h"
#include <iostream>
#include "constants.h"
using namespace std;

/***********************************************************************
   link() - is the constructor that initializes the link object

   Input: next_node -> pointer to the node to which the link will 
                       give frames to
          LINESPEED -> line speed of the link
          PROP_DELAY -> propagation delay of the link
          PL -> loss probability of the link
          PD -> damage probability of the link
*************************************************************************/
link::link(node *next_node, int LINESPEED, double PROP_DELAY, double PL, 
double PD)
{
  linespeed = LINESPEED;
  prop_delay = PROP_DELAY;
  pl = PL;
  pd = PD;
  pok = 1 - (pl+pd);
  nextnode = next_node;
}

/**********************************************************
   randomno() - generates a random number between 0 and 1
     Input: None
     Output: Generated random number
***********************************************************/
double link::randomno()
{
  return (double)rand()/RAND_MAX;
}

/***************************************************************************
processframe() - This function processes the frame that has arrived in the link
                 completely, checks for frame loss, damage and generates 
                 event:frame arriving at node
    input: frameinlink -> pointer to the frame that has arrived in link
           head -> pointer to the eventlist
           sourcelink -> is the link on which the frame has arrived
           current_time is the event time
    output: None
*****************************************************************************/

void link::processframe(frame *frameinlink, eventlist *head, link 
*sourcelink, double current_time)
{
   double randno = randomno();
   
   if((randno>pok) &&(randno<=(pd+pok))) //frame damaged
   {
    if(frameinlink->type == Data)
      cout<<"\nData frame with sequence number "<<frameinlink->dataseq;
    else if(frameinlink->type == ACK)
      cout<<"\nACK frame for sequence "<<frameinlink->ackseq;
    else
      cout<<"\nNACK frame for expected sequence "<<frameinlink->ackseq;
      cout<<" damaged in link.\n";
      frameinlink->status=damaged;
   }

   if(((randno>pok)&&(randno<=(pd+pok)))||(randno<=pok)) //if 
                                  //damaged or ok
   {
           event * newevent2 = new event();
           double traveltime = prop_delay + current_time;
           newevent2->scheduleevent(traveltime, frameinlink, 3, NULL, 
            nextnode, NULL, NULL, sourcelink, NULL);
           PQE *newPQE1 = new PQE(traveltime, 3, newevent2);
           head->enqueue(*newPQE1);
          
   }
   if((randno>(pok+pd))&&(randno<=1))
   {//delete the frame lost in link
     
     if(frameinlink->type == Data)
      cout<<"\nData frame sequence "<<frameinlink->dataseq;
     else if(frameinlink->type == ACK)
      cout<<"\nACK frame for sequence "<<frameinlink->ackseq;
     else
      cout<<"\nNACK frame for expected sequence "<<frameinlink->ackseq;
      cout<<" lost in link.\n";
      delete frameinlink;
      frameinlink = NULL;
   }  
}  
